package de.moritzruth.dracula_musical.device

import de.moritzruth.theaterdsl.device.ColorDV
import de.moritzruth.theaterdsl.device.Device
import de.moritzruth.theaterdsl.device.PercentageDV
import de.moritzruth.theaterdsl.dmx.DmxAddress
import de.moritzruth.theaterdsl.dmx.DmxDataWriter
import de.moritzruth.theaterdsl.dmx.DmxValue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlinx.collections.immutable.persistentListOf
import kotlinx.collections.immutable.persistentSetOf

class StairvilleClb4(override val firstChannel: DmxAddress) : Device {
    override val numberOfChannels: UInt = 14u

    override fun writeDmxData(writer: DmxDataWriter, isLightBehindCurtainOn: Boolean) {
        if (isLightBehindCurtainOn) {
            repeat(3) {
                writer.writeRaw(DmxValue(255u))
                writer.writeRaw(DmxValue(255u))
                writer.writeRaw(DmxValue(255u))
            }

            writer.writeRaw(DmxValue(0u))
            writer.writeRaw(DmxValue(0u))
            writer.writeRaw(DmxValue(0u))

            writer.writeRaw(DmxValue(0u))
            writer.writeRaw(DmxValue(40u))

            return
        }

        for (color in colors) {
            val (red, green, blue) = color.getCurrentValue().getRGB()
            writer.writePercentage(red)
            writer.writePercentage(green)
            writer.writePercentage(blue)
        }

        writer.writePercentage(strobeSpeed.getCurrentValue())
        writer.writePercentage(brightness.getCurrentValue())
    }

    val brightness = PercentageDV(0.percent)
    val strobeSpeed = PercentageDV()

    val color1 = ColorDV(Color(0.degrees, brightness = 0.percent))
    val color2 = ColorDV(Color(0.degrees, brightness = 0.percent))
    val color3 = ColorDV(Color(0.degrees, brightness = 0.percent))
    val color4 = ColorDV(Color(0.degrees, brightness = 0.percent))
    val colors = persistentListOf(color1, color2, color3, color4)

    override val dvs = persistentSetOf(
        brightness,
        strobeSpeed,
        *colors.toTypedArray()
    )
}