package de.moritzruth.dracula_musical.song

import de.moritzruth.dracula_musical.device.*
import de.moritzruth.theaterdsl.show.SceneBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.percent
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

fun SceneBuilderContext.songIrrenhaus() {
    step(StepCue.MusicStart("Irrenhaus", 3.minutes)) {
        actors {
            +"Dr. Sewart"
            // TODO: Expand
            +"Die Irren"
        }

        rightSpotTarget = "Dr. Sewart"

        onRun {
            Spots.right.brightness.fade(100.percent, 5.seconds)
            Washs.both { it.colorWheelMode.static(CoemarProWash.ColorWheelMode.Rotate(30.percent)) }
        }
    }

    lightStep(StepCue.Custom("Musik: Fill-in")) {
        Washs.both { it.brightness.fade(100.percent, 2.seconds) }
        BlinderBars.all {
            it.preset.static(StairvilleSplb.Preset.STUB)  // jumpy, chaotic
            it.brightness.fade(30.percent, 2.seconds)
        }

        Tops.both {
            it.startRoomMovement(8.0)
            it.prismRotationSpeed.static(0.percent)
            it.prismMode.static(FuturelightDmh160.PrismMode.FROST)
            it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.Yellow)
        }
    }

    lightStep(StepCue.Text("Dr. Sewart", "Bei uns ist alles scheißegal", "während letzter Silbe")) {
        FrontLights.all { it.brightness.fade(100.percent, 2.seconds) }
        Washs.both { it.brightness.off(3.seconds) }
        BlinderBars.all { it.brightness.off(3.seconds) }
        backlightBar.brightness.fade(100.percent, 2.seconds)
    }

    lightStep(StepCue.Text("Alle", "Hurra", "letzte Silbe")) {
        FrontLights.all { it.brightness.off() }
        Spots.right.brightness.off()
    }

    lightStep(StepCue.Custom("Nächster Beat")) {
        FrontLights.all { it.brightness.pulseOnce(0.seconds, 0.5.seconds, peak = 100.percent, end = 60.percent) }
        Tops.both { it.brightness.pulseOnce(0.1.seconds, 0.5.seconds, 100.percent) }
        Washs.both { it.brightness.pulseOnce(0.1.seconds, 0.5.seconds, 100.percent) }
        BlinderBars.all {
            it.preset.static(null)
            it.white.static(100.percent)
            it.brightness.pulseOnce(0.1.seconds, 0.5.seconds, 100.percent)
        }
    }

    lightStep(StepCue.Text("Alle", "Wir sind bescheuert!", "Ende")) {
        BlinderBars.all {
            it.preset.static(StairvilleSplb.Preset.STUB)  // jumpy, chaotic
            it.brightness.fade(40.percent, 1.seconds)
        }

        Washs.both { it.brightness.fade(100.percent, 1.seconds) }
        backlightBar.brightness.fade(50.percent, 1.seconds)
    }

    lightStep(StepCue.Custom("Strophe")) {
        Spots.right.brightness.fade(100.percent, 1.seconds)

        BlinderBars.all {
            it.preset.static(StairvilleSplb.Preset.STUB)  // flowy, chaotic
            it.brightness.fade(25.percent, 1.seconds)
        }
    }

    lightStep(StepCue.Text("Dr. Sewart", "Wir wissen, wie das geht", "während letzter Silbe")) {
        FrontLights.all { it.brightness.fade(100.percent, 2.seconds) }

        Washs.both {
            it.strobeSpeed.static(60.percent)
            it.brightness.off(3.seconds)
        }

        backlightBar.strobeSpeed.static(60.percent)
        backlightBar.brightness.fade(100.percent, 2.seconds)

        BlinderBars.all {
            it.presetSpeed.static(80.percent)
            it.brightness.off(3.seconds)
        }
    }

    lightStep(StepCue.Text("Alle", "Hurra", "letzte Silbe")) {
        FrontLights.all { it.brightness.off() }
    }

    lightStep(StepCue.Custom("Nächster Beat")) {
        FrontLights.all { it.brightness.pulseOnce(0.seconds, 0.5.seconds, peak = 100.percent, end = 60.percent) }
        Tops.both { it.brightness.pulseOnce(0.1.seconds, 0.5.seconds, 100.percent) }
        Washs.both { it.brightness.pulseOnce(0.1.seconds, 0.5.seconds, 100.percent) }
        BlinderBars.all {
            it.preset.static(null)
            it.brightness.pulseOnce(0.1.seconds, 0.5.seconds, 100.percent)
        }
    }

    lightStep(StepCue.Text("Alle", "Wir sind bescheuert!", "Ende")) {
        BlinderBars.all {
            it.preset.static(StairvilleSplb.Preset.STUB)  // jumpy, chaotic
            it.brightness.fade(40.percent, 1.seconds)
        }

        Washs.both { it.brightness.fade(100.percent, 1.seconds) }
        backlightBar.brightness.fade(50.percent, 1.seconds)
    }

    lightStep(StepCue.Custom("Strophe")) {
        Spots.right.brightness.fade(100.percent, 1.seconds)

        BlinderBars.all {
            it.preset.static(StairvilleSplb.Preset.STUB)  // flowy, chaotic
            it.brightness.fade(25.percent, 1.seconds)
        }
    }

    lightStep(StepCue.Text("Dr. Sewart", "Wir warten schon auf dich!", "während letzter Silbe")) {
        FrontLights.all { it.brightness.fade(100.percent, 2.seconds) }

        Washs.both {
            it.strobeSpeed.static(60.percent)
            it.brightness.off(3.seconds)
        }

        backlightBar.strobeSpeed.static(60.percent)
        backlightBar.brightness.fade(100.percent, 2.seconds)

        BlinderBars.all {
            it.presetSpeed.static(80.percent)
            it.brightness.off(3.seconds)
        }
    }

    lightStep(StepCue.Text("Alle", "Hurra", "letzte Silbe")) {
        FrontLights.all { it.brightness.off() }
        backlightBar.strobeSpeed.off()
    }

    lightStep(StepCue.Custom("Nächster Beat")) {
        FrontLights.all { it.brightness.pulseOnce(0.seconds, 0.5.seconds, peak = 100.percent, end = 60.percent) }
        Tops.both { it.brightness.pulseOnce(0.1.seconds, 0.5.seconds, 100.percent) }
        Washs.both { it.brightness.pulseOnce(0.1.seconds, 0.5.seconds, 100.percent) }
        BlinderBars.all {
            it.preset.static(null)
            it.brightness.pulseOnce(0.1.seconds, 0.5.seconds, 100.percent)
        }
    }

    lightStep(StepCue.Text("Alle", "Wir sind bescheuert!", "Ende")) {
        BlinderBars.all {
            it.preset.static(StairvilleSplb.Preset.STUB)  // jumpy, chaotic
            it.brightness.fade(40.percent, 1.seconds)
        }

        Washs.both { it.brightness.fade(100.percent, 1.seconds) }
        backlightBar.brightness.fade(50.percent, 1.seconds)

        Tops.both {
            it.pointAtStageMiddleSingleCenterMarking()
        }
    }

    lightStep(StepCue.Text("Alle", "Gott sei Dank!")) {
        Washs.both {
            it.brightness.off(0.5.seconds)
            it.pointAtAudience()
            it.colorWheelMode.static(CoemarProWash.ColorWheelMode.WarmWhite)
        }
    }

    lightStep(StepCue.Custom("Schlussschlag")) {
        FrontLights.all { it.brightness.static(100.percent) }
        Tops.both { it.brightness.pulseOnce(0.1.seconds, 2.seconds, 100.percent) }
        Washs.both { it.brightness.pulseOnce(0.1.seconds, 2.seconds, 100.percent) }
        BlinderBars.all {
            it.preset.static(null)
            it.color.static(Color.WHITE)
            it.brightness.pulseOnce(0.1.seconds, 2.seconds, 100.percent)
        }
    }

    step(StepCue.MusicEnd) {
        rightSpotTarget = null

        onRun {
            FrontLights.all { it.brightness.fade(40.percent, 5.seconds) }
            Spots.right.brightness.off(5.seconds)
        }
    }
}