package de.moritzruth.theaterdsl.show

import de.moritzruth.theaterdsl.util.DurationAsMillisecondsSerializer
import de.moritzruth.theaterdsl.util.InstantAsEpochMillisecondsSerializer
import kotlinx.datetime.Instant
import kotlinx.serialization.Serializable
import kotlin.time.Duration

@Serializable
data class Show(val acts: List<Act>) {
    fun getNextValidPosition(position: ShowPosition): ShowPosition? {
        if (position == ShowPosition.START) return ShowPosition.FIRST

        var nextStep = position.step + 1
        var nextScene = position.scene
        var nextAct = position.act

        if (nextStep >= acts[position.act].scenes[position.scene].steps.size) {
            nextStep = 0
            nextScene++

            if (nextScene >= acts[position.act].scenes.size) {
                nextScene = 0
                nextAct++

                if (nextAct >= acts.size) {
                    return null
                }
            }
        }

        return ShowPosition(nextAct, nextScene, nextStep)
    }
}

@Serializable
data class ShowPosition(val act: Int, val scene: Int, val step: Int) : Comparable<ShowPosition> {
    companion object {
        val START = ShowPosition(-1, 0, 0)
        val FIRST = ShowPosition(0, 0, 0)
    }

    override fun compareTo(other: ShowPosition): Int {
        return if (act == other.act) {
            if (scene == other.scene) step.compareTo(other.step)
            else scene.compareTo(other.scene)
        } else act.compareTo(other.act)
    }
}

@Serializable
data class ShowState(
    val position: ShowPosition,
    val message: String,
    val activeMusic: ActiveShowMusic?,
    @Serializable(with = InstantAsEpochMillisecondsSerializer::class)
    val musicStartTime: Instant,
    val isLightBehindCurtainOn: Boolean
)

@Serializable
data class ActiveShowMusic(
    val title: String,
    @Serializable(with = DurationAsMillisecondsSerializer::class)
    val duration: Duration
)
