import { Ref, ComputedRef, computed, UnwrapRef, ref, watchEffect } from "vue"

export const computedIfPresent = <T, V>(object: Ref<T>, access: (value: Exclude<T, undefined | null>) => V): ComputedRef<V | undefined | null> => computed(() => {
  if (object.value === null) return null
  else if (object.value === undefined) return undefined

  return access(object.value as Exclude<T, undefined | null>)
})

export const computedOrNull = <T, V>(object: Ref<T | null>, access: (value: T) => V): ComputedRef<V | null> => computed(() => {
  if (object.value === null) return null
  return access(object.value)
})

export function avoidNull<T>(originRef: Ref<UnwrapRef<T> | null>): ComputedRef<UnwrapRef<T> | null> {
  const nullAvoidingRef = ref<T | null>(null)

  watchEffect(() => {
    if (originRef.value !== null) nullAvoidingRef.value = originRef.value
  })

  return computed(() => nullAvoidingRef.value)
}

export function formatSeconds(seconds: number) {
  const duration = new Date(seconds * 1000)
  return `${duration.getMinutes().toFixed()}:${duration.getSeconds().toFixed().padStart(2, '0')}`
}
